@@ -7,14 +7,14 @@ exports.port = 5858;
 
 exports.start = function() {
   if (process.argv.length < 3) {
-    console.error("Usage: node debug script.js");
+    console.error('Usage: node debug script.js');
     process.exit(1);
   }
 
   var interface = new Interface();
-  process.on('uncaughtException', function (e) {
+  process.on('uncaughtException', function(e) {
     console.error("There was an internal error in Node's debugger. " +
-                  "Please report this bug.");
+        'Please report this bug.');
     console.error(e.message);
     console.error(e.stack);
     if (interface.child) interface.child.kill();
@@ -141,7 +141,7 @@ exports.Client = Client;
 
 Client.prototype._addHandle = function(desc) {
   if (typeof desc != 'object' || typeof desc.handle != 'number') {
-    throw new Error('bad type');
+    return;
   }
 
   this.handles[desc.handle] = desc;
@@ -260,7 +260,13 @@ Client.prototype.reqEval = function(expression, cb) {
   }
 
   // Otherwise we need to get the current frame to see which scopes it has.
-  this.reqBacktrace(function (bt) {
+  this.reqBacktrace(function(bt) {
+    if (!bt.frames) {
+      // ??
+      cb({});
+      return;
+    }
+
     var frame = bt.frames[self.currentFrame];
 
     var evalFrames = frame.scopes.map(function(s) {
@@ -414,7 +420,7 @@ Client.prototype.mirrorObject = function(handle, cb) {
 
     this.reqLookup(propertyRefs, function(res) {
       if (!res.success) {
-        console.error("problem with reqLookup");
+        console.error('problem with reqLookup');
         if (cb) cb(handle);
         return;
       }
@@ -428,7 +434,13 @@ Client.prototype.mirrorObject = function(handle, cb) {
 
       for (var i = 0; i < handle.properties.length; i++) {
         var value = res.body[handle.properties[i].ref];
-        var mirrorValue = value.value ? value.value : value.text;
+        var mirrorValue;
+        if (value) {
+          mirrorValue = value.value ? value.value : value.text;
+        } else {
+          mirrorValue = '[?]';
+        }
+
 
         if (Array.isArray(mirror) &&
             typeof handle.properties[i].name != 'number') {
@@ -479,7 +491,7 @@ Client.prototype.fullTrace = function(cb) {
       //   column: 2,
       //   sourceLineText: '  debugger;',
       //   scopes: [ { type: 1, index: 0 }, { type: 0, index: 1 } ],
-      //   text: '#00 blah() /home/ryan/projects/node/test-debug.js line 8 column 3 (position 161)' }
+      //   text: '#00 blah() /home/ryan/projects/node/test-debug.js l...' }
       refs.push(frame.script.ref);
       refs.push(frame.func.ref);
       refs.push(frame.receiver.ref);
@@ -496,7 +508,7 @@ Client.prototype.fullTrace = function(cb) {
       if (cb) cb(trace);
     });
   });
-}
+};
 
 
 
@@ -516,7 +528,7 @@ var commands = [
   'run',
   'scripts',
   'step',
-  'version',
+  'version'
 ];
 
 
@@ -565,22 +577,21 @@ function SourceInfo(body) {
 // "node debug"
 function Interface() {
   var self = this;
-  var term = this.term =
-    readline.createInterface(process.stdout, function (line) {
-      return self.complete(line);
-    });
   var child;
   var client;
-  var term;
+
+  function complete(line) {
+    return self.complete(line);
+  }
+
+  var term = readline.createInterface(process.stdin, process.stdout, complete);
+  this.term = term;
 
   process.on('exit', function() {
     self.killChild();
   });
 
   this.stdin = process.openStdin();
-  this.stdin.addListener('data', function(chunk) {
-    term.write(chunk);
-  });
 
   term.setPrompt('debug> ');
   term.prompt();
