@@ -165,6 +165,48 @@ Socket.prototype.setTTL = function(arg) {
   }
 };
 
+Socket.prototype.setMulticastTTL = function(arg) {
+  var newttl = parseInt(arg);
+
+  if (newttl >= 0 && newttl < 256) {
+    return binding.setMulticastTTL(this.fd, newttl);
+  } else {
+    throw new Error('New MulticastTTL must be between 0 and 255');
+  }
+};
+
+Socket.prototype.setMulticastLoopback = function(arg) {
+  if (arg) {
+    return binding.setMulticastLoopback(this.fd, 1);
+  } else {
+    return binding.setMulticastLoopback(this.fd, 0);
+  }
+};
+
+Socket.prototype.addMembership = function(multicastAddress,
+                                          multicastInterface) {
+  var self = this;
+  dnsLookup(this.type, multicastAddress, function(err, ip, addressFamily) {
+    if (err) {  // DNS error
+      self.emit('error', err);
+      return;
+    }
+    binding.addMembership(self.fd, multicastAddress, multicastInterface);
+  });
+};
+
+Socket.prototype.dropMembership = function(multicastAddress,
+                                           multicastInterface) {
+  var self = this;
+  dnsLookup(this.type, multicastAddress, function(err, ip, addressFamily) {
+    if (err) {  // DNS error
+      self.emit('error', err);
+      return;
+    }
+    binding.dropMembership(self.fd, multicastAddress, multicastInterface);
+  });
+};
+
 // translate arguments from JS API into C++ API, possibly after DNS lookup
 Socket.prototype.send = function(buffer, offset, length) {
   var self = this;
