@@ -185,11 +185,14 @@ function modeNum(m, def) {
 }
 
 fs.open = function(path, flags, mode, callback) {
-  mode = modeNum(mode, '0666');
-  var callback_ = arguments[arguments.length - 1];
-  var callback = (typeof(callback_) == 'function' ? callback_ : null);
+  callback = arguments[arguments.length - 1];
+  if (typeof(callback) !== 'function') {
+    callback = noop;
+  }
 
-  binding.open(path, stringToFlags(flags), mode, callback || noop);
+  mode = modeNum(mode, '0666');
+
+  binding.open(path, stringToFlags(flags), mode, callback);
 };
 
 fs.openSync = function(path, flags, mode) {
@@ -918,6 +921,9 @@ ReadStream.prototype.resume = function() {
     this.buffer = null;
   }
 
+  // hasn't opened yet.
+  if (null == this.fd) return;
+
   this._read();
 };
 
