@@ -303,9 +303,6 @@ function OutgoingMessage() {
   this._trailer = '';
 
   this.finished = false;
-  
-  this._headers = {};
-  this._headerNames = {};
 }
 util.inherits(OutgoingMessage, stream.Stream);
 
@@ -507,6 +504,8 @@ OutgoingMessage.prototype.setHeader = function(name, value) {
   }
 
   var key = name.toLowerCase();
+  this._headers = this._headers || {};
+  this._headerNames = this._headerNames || {};
   this._headers[key] = value;
   this._headerNames[key] = name;
 };
@@ -521,6 +520,8 @@ OutgoingMessage.prototype.getHeader = function(name) {
     throw new Error("Can't use mutable header APIs after sent.");
   }
 
+  if (!this._headers) return;
+
   var key = name.toLowerCase();
   return this._headers[key];
 };
@@ -535,6 +536,8 @@ OutgoingMessage.prototype.removeHeader = function(name) {
     throw new Error("Can't remove headers after they are sent.");
   }
 
+  if (!this._headers) return;
+
   var key = name.toLowerCase();
   delete this._headers[key];
   delete this._headerNames[key];
@@ -545,6 +548,9 @@ OutgoingMessage.prototype._renderHeaders = function() {
   if (this._header) {
     throw new Error("Can't render headers after they are sent to the client.");
   }
+
+  if (!this._headers) return {};
+
   var headers = {};
   var keys = Object.keys(this._headers);
   for (var i = 0, l = keys.length; i < l; i++) {
@@ -754,7 +760,7 @@ ServerResponse.prototype.writeContinue = function() {
 };
 
 ServerResponse.prototype._implicitHeader = function() {
-  this.writeHead(this.statusCode, this._renderHeaders());
+  this.writeHead(this.statusCode);
 };
 
 ServerResponse.prototype.writeHead = function(statusCode) {
@@ -768,10 +774,36 @@ ServerResponse.prototype.writeHead = function(statusCode) {
     headerIndex = 1;
   }
 
-  if (typeof arguments[headerIndex] == 'object') {
-    headers = arguments[headerIndex];
+  var obj = arguments[headerIndex];
+
+  if (obj && this._headers) {
+    // Slow-case: when progressive API and header fields are passed.
+    headers = this._renderHeaders();
+
+    if (Array.isArray(obj)) {
+      // handle array case
+      // TODO: remove when array is no longer accepted
+      var field;
+      for (var i = 0, len = obj.length; i < len; ++i) {
+        field = obj[i][0];
+        if (field in headers) {
+          obj.push([field, headers[field]]);
+        }
+      }
+      headers = obj;
+
+    } else {
+      // handle object case
+      for (var k in obj) {
+        if (k) headers[k] = obj[k];
+      }
+    }
+  } else if (this._headers) {
+    // only progressive api is used
+    headers = this._renderHeaders();
   } else {
-    headers = {};
+    // only writeHead() called
+    headers = obj;
   }
 
   var statusLine = 'HTTP/1.1 ' + statusCode.toString() + ' ' +
@@ -977,13 +1009,13 @@ function connectionListener(socket) {
 
     if (!self.httpAllowHalfOpen) {
       abortIncoming();
-      socket.end();
+      if (socket.writable) socket.end();
     } else if (outgoing.length) {
       outgoing[outgoing.length - 1]._last = true;
     } else if (socket._httpMessage) {
       socket._httpMessage._last = true;
     } else {
-      socket.end();
+      if (socket.writable) socket.end();
     }
   };
 
@@ -1054,7 +1086,7 @@ exports._connectionListener = connectionListener;
 function Agent(options) {
   this.options = options;
   this.host = options.host;
-  this.port = options.port;
+  this.port = options.port || 80;
 
   this.queue = [];
   this.sockets = [];
@@ -1112,6 +1144,10 @@ Agent.prototype._establishNewConnection = function() {
 
   this.sockets.push(socket);
 
+  // Cycle so the request can be assigned to this new socket.
+  self._cycle();
+  assert(socket._httpMessage);
+
   // Add a parser to the socket.
   var parser = parsers.alloc();
   parser.reinitialize('response');
@@ -1134,6 +1170,13 @@ Agent.prototype._establishNewConnection = function() {
 
     req.emit('error', err);
     req._hadError = true; // hacky
+
+    // clean up so that agent can handle new requests
+    parser.finish();
+    socket.destroy();
+    self._removeSocket(socket);
+    parsers.free(parser);
+    self._cycle();
   });
 
   socket.ondata = function(d, start, end) {
@@ -1181,6 +1224,7 @@ Agent.prototype._establishNewConnection = function() {
 
   // When the socket closes remove it from the list of available sockets.
   socket.on('close', function() {
+    debug('AGENT socket close');
     // This is really hacky: What if someone issues a request, the server
     // accepts, but then terminates the connection. There is no parse error,
     // there is no socket-level error. How does the user get informed?
@@ -1205,9 +1249,15 @@ Agent.prototype._establishNewConnection = function() {
   parser.onIncoming = function(res, shouldKeepAlive) {
     debug('AGENT incoming response!');
 
-    var req = socket._httpMessage;
-    assert(req);
+    // If we're receiving a message but we don't have a corresponding
+    // request - then somehow the server is seriously messed up and sending
+    // multiple responses at us. In this case we'll just bail.
+    if (!socket._httpMessage) {
+      socket.destroy();
+      return;
+    }
 
+    var req = socket._httpMessage;
     req.res = res;
 
     // Responses to HEAD requests are AWFUL. Ask Ryan.
@@ -1232,7 +1282,8 @@ Agent.prototype._establishNewConnection = function() {
       // but outgoingFlush instead.
       if (!req.shouldKeepAlive) {
         debug('AGENT socket.end()');
-        socket.end();
+        if (socket.writable) socket.end();
+        assert(!socket.writable);
       } else {
         debug('AGENT socket keep-alive');
       }
@@ -1266,22 +1317,22 @@ Agent.prototype._getConnection = function(host, port, cb) {
 // waiting sockets. If a waiting socket cannot be found, it will
 // start the process of establishing one.
 Agent.prototype._cycle = function() {
-  debug('Agent _cycle sockets=' + this.sockets.length +
-        ' queue=' + this.queue.length);
+  debug('Agent _cycle sockets=' + this.sockets.length + ' queue=' + this.queue.length);
   var self = this;
 
   var first = this.queue[0];
   if (!first) return;
 
-
   var haveConnectingSocket = false;
 
   // First try to find an available socket.
   for (var i = 0; i < this.sockets.length; i++) {
     var socket = this.sockets[i];
     // If the socket doesn't already have a message it's sending out
-    // and the socket is available for writing...
-    if (!socket._httpMessage && (socket.writable && socket.readable)) {
+    // and the socket is available for writing or it's connecting.
+    // In particular this rules out sockets that are closing.
+    if (!socket._httpMessage &&
+        ((socket.writable && socket.readable) || socket._httpConnecting)) {
       debug('Agent found socket, shift');
       // We found an available connection!
       this.queue.shift(); // remove first from queue.
@@ -1317,6 +1368,7 @@ var agents = {};
 
 
 function getAgent(host, port) {
+  port = port || 80;
   var id = host + ':' + port;
   var agent = agents[id];
 
