@@ -85,7 +85,8 @@ function tryPackage(requestPath, exts) {
   if (!pkg || !pkg.main) return false;
 
   var filename = path.resolve(requestPath, pkg.main);
-  return tryFile(filename) || tryExtensions(filename, exts);
+  return tryFile(filename) || tryExtensions(filename, exts) ||
+         tryExtensions(path.resolve(filename, 'index'), exts);
 }
 
 // In order to minimize unnecessary lstat() calls,
