@@ -13,28 +13,44 @@ var EventEmitter = require('events').EventEmitter;
 var tty = require('tty');
 
 
-exports.createInterface = function(output, completer) {
-  return new Interface(output, completer);
+exports.createInterface = function(input, output, completer) {
+  return new Interface(input, output, completer);
 };
 
 
-function Interface(output, completer) {
-  if (!(this instanceof Interface)) return new Interface(output, completer);
+function Interface(input, output, completer) {
+  if (!(this instanceof Interface)) {
+    return new Interface(input, output, completer);
+  }
   EventEmitter.call(this);
 
+  var self = this;
+
   this.output = output;
+  this.input = input;
+  input.resume();
+
   this.completer = completer;
 
   this.setPrompt('> ');
 
-  this.enabled = tty.isatty(output.fd);
+  this.enabled = output.isTTY;
 
   if (parseInt(process.env['NODE_NO_READLINE'], 10)) {
     this.enabled = false;
   }
 
-  if (this.enabled) {
-    // input refers to stdin
+  if (!this.enabled) {
+    input.on('data', function(data) {
+      self._normalWrite(data);
+    });
+
+  } else {
+
+    // input usually refers to stdin
+    input.on('keypress', function(s, key) {
+      self._ttyWrite(s, key);
+    });
 
     // Current line
     this.line = '';
@@ -49,13 +65,12 @@ function Interface(output, completer) {
     this.history = [];
     this.historyIndex = -1;
 
-    // 0 for stdin
-    var winSize = tty.getWindowSize(0);
+    var winSize = tty.getWindowSize(output.fd);
     exports.columns = winSize[1];
 
     if (process.listeners('SIGWINCH').length === 0) {
       process.on('SIGWINCH', function() {
-        var winSize = tty.getWindowSize(0);
+        var winSize = tty.getWindowSize(output.fd);
         exports.columns = winSize[1];
       });
     }
@@ -139,17 +154,17 @@ Interface.prototype._refreshLine = function() {
   if (this._closed) return;
 
   // Cursor to left edge.
-  this.output.write('\x1b[0G');
+  this.output.cursorTo(0);
 
   // Write the prompt and the current buffer content.
   this.output.write(this._prompt);
   this.output.write(this.line);
 
   // Erase to right.
-  this.output.write('\x1b[0K');
+  this.output.clearLine(1);
 
   // Move cursor to original position.
-  this.output.write('\x1b[0G\x1b[' + (this._promptLength + this.cursor) + 'C');
+  this.output.cursorTo(this._promptLength + this.cursor);
 };
 
 
@@ -178,16 +193,17 @@ Interface.prototype.resume = function() {
 };
 
 
-Interface.prototype.write = function(d) {
+Interface.prototype.write = function(d, key) {
   if (this._closed) return;
-  return this.enabled ? this._ttyWrite(d) : this._normalWrite(d);
+  this.enabled ? this._ttyWrite(d, key) : this._normalWrite(d, key);
 };
 
 
 Interface.prototype._normalWrite = function(b) {
   // Very simple implementation right now. Should try to break on
   // new lines.
-  this._onLine(b.toString());
+  if (b !== undefined)
+    this._onLine(b.toString());
 };
 
 Interface.prototype._insertString = function(c) {
@@ -274,6 +290,7 @@ Interface.prototype._tabComplete = function() {
   }
 };
 
+
 function commonPrefix(strings) {
   if (!strings || strings.length == 0) {
     return '';
@@ -289,6 +306,88 @@ function commonPrefix(strings) {
   return min;
 }
 
+
+Interface.prototype._wordLeft = function() {
+  if (this.cursor > 0) {
+    var leading = this.line.slice(0, this.cursor);
+    var match = leading.match(/([^\w\s]+|\w+|)\s*$/);
+    this.cursor -= match[0].length;
+    this._refreshLine();
+  }
+}
+
+
+Interface.prototype._wordRight = function() {
+  if (this.cursor < this.line.length) {
+    var trailing = this.line.slice(this.cursor);
+    var match = trailing.match(/^(\s+|\W+|\w+)\s*/);
+    this.cursor += match[0].length;
+    this._refreshLine();
+  }
+}
+
+
+Interface.prototype._deleteLeft = function() {
+  if (this.cursor > 0 && this.line.length > 0) {
+    this.line = this.line.slice(0, this.cursor - 1) +
+                this.line.slice(this.cursor, this.line.length);
+
+    this.cursor--;
+    this._refreshLine();
+  }
+};
+
+
+Interface.prototype._deleteRight = function() {
+  this.line = this.line.slice(0, this.cursor) +
+              this.line.slice(this.cursor + 1, this.line.length);
+  this._refreshLine();
+};
+
+
+Interface.prototype._deleteWordLeft = function() {
+  if (this.cursor > 0) {
+    var leading = this.line.slice(0, this.cursor);
+    var match = leading.match(/([^\w\s]+|\w+|)\s*$/);
+    leading = leading.slice(0, leading.length - match[0].length);
+    this.line = leading + this.line.slice(this.cursor, this.line.length);
+    this.cursor = leading.length;
+    this._refreshLine();
+  }
+}
+
+
+Interface.prototype._deleteWordRight = function() {
+  if (this.cursor < this.line.length) {
+    var trailing = this.line.slice(this.cursor);
+    var match = trailing.match(/^(\s+|\W+|\w+)\s*/);
+    this.line = this.line.slice(0, this.cursor) +
+                trailing.slice(match[0].length);
+    this._refreshLine();
+  }
+}
+
+
+Interface.prototype._deleteLineLeft = function() {
+  this.line = this.line.slice(this.cursor);
+  this.cursor = 0;
+  this._refreshLine();
+}
+
+
+Interface.prototype._deleteLineRight = function() {
+  this.line = this.line.slice(0, this.cursor);
+  this._refreshLine();
+}
+
+
+Interface.prototype._line = function() {
+  var line = this._addHistory();
+  this.output.write('\r\n');
+  this._onLine(line);
+};
+
+
 Interface.prototype._historyNext = function() {
   if (this.historyIndex > 0) {
     this.historyIndex--;
@@ -304,6 +403,7 @@ Interface.prototype._historyNext = function() {
   }
 };
 
+
 Interface.prototype._historyPrev = function() {
   if (this.historyIndex + 1 < this.history.length) {
     this.historyIndex++;
@@ -326,223 +426,206 @@ Interface.prototype._attemptClose = function() {
 
 
 // handle a write from the tty
-Interface.prototype._ttyWrite = function(b) {
-  switch (b[0]) {
-    /* ctrl+c */
-    case 3:
-      //process.kill(process.pid, "SIGINT");
-      if (this.listeners('SIGINT').length) {
-        this.emit('SIGINT');
-      } else {
-        // default behavior, end the readline
-        this._attemptClose();
-      }
-      break;
+Interface.prototype._ttyWrite = function(s, key) {
+  var next_word, next_non_word, previous_word, previous_non_word;
+  key = key || {};
 
-    case 4: // control-d, delete right or EOF
-      if (this.cursor === 0 && this.line.length === 0) {
-        this._attemptClose();
-      } else if (this.cursor < this.line.length) {
-        this.line = this.line.slice(0, this.cursor) +
-                    this.line.slice(this.cursor + 1, this.line.length);
+  if (key.ctrl && key.shift) {
+    /* Control and shift pressed */
+    switch (key.name) {
+      case "backspace":
+        this._deleteLineLeft();
+        break;
 
-        this._refreshLine();
-      }
-      break;
+      case "delete":
+        this._deleteLineRight();
+        break;
+    }
 
-    case 13:    /* enter */
-      var line = this._addHistory();
-      this.output.write('\r\n');
-      this._onLine(line);
-      break;
+  } else if (key.ctrl) {
+    /* Control key pressed */
 
-    case 127:   /* backspace */
-    case 8:     /* ctrl+h */
-      if (this.cursor > 0 && this.line.length > 0) {
-        this.line = this.line.slice(0, this.cursor - 1) +
-                    this.line.slice(this.cursor, this.line.length);
-
-        this.cursor--;
-        this._refreshLine();
-      }
-      break;
-
-    case 21: /* Ctrl+u, delete the whole line. */
-      this.cursor = 0;
-      this.line = '';
-      this._refreshLine();
-      break;
-
-    case 11: /* Ctrl+k, delete from current to end of line. */
-      this.line = this.line.slice(0, this.cursor);
-      this._refreshLine();
-      break;
-
-    case 1: /* Ctrl+a, go to the start of the line */
-      this.cursor = 0;
-      this._refreshLine();
-      break;
-
-    case 5: /* ctrl+e, go to the end of the line */
-      this.cursor = this.line.length;
-      this._refreshLine();
-      break;
-
-    case 2: // control-b, back one character
-      if (this.cursor > 0) {
-        this.cursor--;
-        this._refreshLine();
-      }
-      break;
-
-    case 6: // control-f, forward one character
-      if (this.cursor != this.line.length) {
-        this.cursor++;
-        this._refreshLine();
-      }
-      break;
-
-    case 14: // control-n, next history item
-      this._historyNext();
-      break;
-
-    case 23: // control-w, delete backwards to a word boundary
-      if (this.cursor !== 0) {
-        var leading = this.line.slice(0, this.cursor);
-        var match = leading.match(/\s?((\W+|\w+)\s*)$/);
-        leading = leading.slice(0, leading.length - match[1].length);
-        this.line = leading + this.line.slice(this.cursor, this.line.length);
-        this.cursor = leading.length;
-        this._refreshLine();
-      }
-      break;
-
-    case 9: // tab, completion
-      if (this.completer) {
-        this._tabComplete();
-      }
-      break;
-
-    case 16: // control-p, previous history item
-      this._historyPrev();
-      break;
-
-    case 26:    /* ctrl+z */
-      process.kill(process.pid, 'SIGTSTP');
-      return;
-
-    case 27:    /* escape sequence */
-      var next_word, next_non_word, previous_word, previous_non_word;
-
-      if (b[1] === 98 && this.cursor > 0) {
-        // meta-b - backward word
-        previous_word = this.line.slice(0, this.cursor)
-              .split('').reverse().join('')
-              .search(/\w/);
-        if (previous_word !== -1) {
-          previous_non_word = this.line.slice(0, this.cursor - previous_word)
-              .split('').reverse().join('')
-              .search(/\W/);
-          if (previous_non_word !== -1) {
-            this.cursor -= previous_word + previous_non_word;
-            this._refreshLine();
-            break;
-          }
+    switch (key.name) {
+      case 'c':
+        if (this.listeners('SIGINT').length) {
+          this.emit('SIGINT');
+        } else {
+          // default behavior, end the readline
+          this._attemptClose();
         }
+        break;
+
+      case 'h': // delete left
+        this._deleteLeft();
+        break;
+
+      case 'd': // delete right or EOF
+        if (this.cursor === 0 && this.line.length === 0) {
+          this._attemptClose();
+        } else if (this.cursor < this.line.length) {
+          this._deleteRight();
+        }
+        break;
+
+      case 'u': // delete the whole line
+        this.cursor = 0;
+        this.line = '';
+        this._refreshLine();
+        break;
+
+      case 'k': // delete from current to end of line
+        this._deleteLineRight();
+        break;
+
+      case 'a': // go to the start of the line
         this.cursor = 0;
         this._refreshLine();
+        break;
 
-      } else if (b[1] === 102 && this.cursor < this.line.length) {
-        // meta-f - forward word
-        next_word = this.line.slice(this.cursor, this.line.length).search(/\w/);
-        if (next_word !== -1) {
-          next_non_word = this.line.slice(this.cursor + next_word,
-                                          this.line.length).search(/\W/);
-          if (next_non_word !== -1) {
-            this.cursor += next_word + next_non_word;
-            this._refreshLine();
-            break;
-          }
-        }
+      case 'e': // go to the end of the line
         this.cursor = this.line.length;
         this._refreshLine();
+        break;
 
-      } else if (b[1] === 100 && this.cursor < this.line.length) {
-        // meta-d delete forward word
-        next_word = this.line.slice(this.cursor, this.line.length).search(/\w/);
-        if (next_word !== -1) {
-          next_non_word = this.line.slice(this.cursor + next_word,
-                                          this.line.length).search(/\W/);
-          if (next_non_word !== -1) {
-            this.line = this.line.slice(this.cursor +
-                                        next_word +
-                                        next_non_word);
-            this.cursor = 0;
-            this._refreshLine();
-            break;
-          }
-        }
-        this.line = '';
-        this.cursor = 0;
-        this._refreshLine();
-
-      } else if (b[1] === 91 && b[2] === 68) {
-        // left arrow
+      case 'b': // back one character
         if (this.cursor > 0) {
           this.cursor--;
-          this.output.write('\x1b[0D');
+          this._refreshLine();
         }
+        break;
 
-      } else if (b[1] === 91 && b[2] === 67) {
-        // right arrow
+      case 'f': // forward one character
         if (this.cursor != this.line.length) {
           this.cursor++;
-          this.output.write('\x1b[0C');
+          this._refreshLine();
         }
+        break;
 
-      } else if ((b[1] === 91 && b[2] === 72) ||
-                 (b[1] === 79 && b[2] === 72) ||
-                 (b[1] === 91 && b[2] === 55) ||
-                 (b[1] === 91 && b[2] === 49 && (b[3] && b[3] === 126))) {
-        // home
+      case 'n': // next history item
+        this._historyNext();
+        break;
+
+      case 'p': // previous history item
+        this._historyPrev();
+        break;
+
+      case 'z':
+        process.kill(process.pid, 'SIGTSTP');
+        return;
+
+      case 'w': // delete backwards to a word boundary
+      case 'backspace':
+        this._deleteWordLeft();
+        break;
+
+      case 'delete': // delete forward to a word boundary
+        this._deleteWordRight();
+        break;
+
+      case 'backspace':
+        this._deleteWordLeft();
+        break;
+
+      case 'left':
+        this._wordLeft();
+        break;
+
+      case 'right':
+        this._wordRight();
+    }
+
+  } else if (key.meta) {
+    /* Meta key pressed */
+
+    switch (key.name) {
+      case 'b': // backward word
+        this._wordLeft();
+        break;
+
+      case 'f': // forward word
+        this._wordRight();
+        break;
+
+      case 'd': // delete forward word
+      case 'delete':
+        this._deleteWordRight();
+        break;
+
+      case 'backspace': // delete backwards to a word boundary
+        this._deleteWordLeft();
+        break;
+    }
+
+  } else {
+    /* No modifier keys used */
+
+    switch (key.name) {
+      case 'enter':
+        this._line();
+        break;
+
+      case 'backspace':
+        this._deleteLeft();
+        break;
+
+      case 'delete':
+        this._deleteRight();
+        break;
+
+      case 'tab': // tab completion
+        if (this.completer) {
+          this._tabComplete();
+        }
+        break;
+
+      case 'left':
+        if (this.cursor > 0) {
+          this.cursor--;
+          this.output.moveCursor(-1, 0);
+        }
+        break;
+
+      case 'right':
+        if (this.cursor != this.line.length) {
+          this.cursor++;
+          this.output.moveCursor(1, 0);
+        }
+        break;
+
+      case 'home':
         this.cursor = 0;
         this._refreshLine();
-      } else if ((b[1] === 91 && b[2] === 70) ||
-                 (b[1] === 79 && b[2] === 70) ||
-                 (b[1] === 91 && b[2] === 56) ||
-                 (b[1] === 91 && b[2] === 52 && (b[3] && b[3] === 126))) {
-        // end
+        break;
+
+      case 'end':
         this.cursor = this.line.length;
         this._refreshLine();
+        break;
 
-      } else if (b[1] === 91 && b[2] === 65) {
-        // up arrow
+      case 'up':
         this._historyPrev();
+        break;
 
-      } else if (b[1] === 91 && b[2] === 66) {
-        // down arrow
+      case 'down':
         this._historyNext();
+        break;
 
-      } else if (b[1] === 91 && b[2] === 51 && this.cursor < this.line.length) {
-        // delete right
-        this.line = this.line.slice(0, this.cursor) +
-                    this.line.slice(this.cursor + 1, this.line.length);
-        this._refreshLine();
+      default:
+        if (Buffer.isBuffer(s))
+          s = s.toString('utf-8');
 
-      }
-      break;
-
-    default:
-      var c = b.toString('utf8');
-      var lines = c.split(/\r\n|\n|\r/);
-      for (var i = 0, len = lines.length; i < len; i++) {
-        if (i > 0) {
-          this._ttyWrite(new Buffer([13]));
+        if (s) {
+          var lines = s.split(/\r\n|\n|\r/);
+          for (var i = 0, len = lines.length; i < len; i++) {
+            if (i > 0) {
+              this._line();
+            }
+            this._insertString(lines[i]);
+          }
         }
-        this._insertString(lines[i]);
-      }
-      break;
+    }
   }
 };
 
+
 exports.Interface = Interface;
