@@ -69,7 +69,9 @@ CryptoStream.prototype.write = function(data /* , encoding, cb */) {
   this._pending.push(data);
   this._pendingCallbacks.push(cb);
 
+  this.pair._writeCalled = true;
   this.pair._cycle();
+
   return this._writeState;
 };
 
@@ -507,6 +509,11 @@ SecurePair.prototype._cycle = function() {
     return;
   }
 
+  // Make this function reentrant.
+  if (this._cycleLock) return;
+  this._cycleLock = true;
+  this._writeCalled = false;
+
   var established = this._secureEstablished;
 
   this.encrypted._pull();
@@ -514,8 +521,15 @@ SecurePair.prototype._cycle = function() {
   this.cleartext._push();
   this.encrypted._push();
 
-  if (!established && this._secureEstablished) {
+  this._cycleLock = false;
+
+  if (this._done) {
+    return;
+  }
+
+  if ((!established && this._secureEstablished) || this._writeCalled) {
     // If we were not established but now we are, let's cycle again.
+    // Or if there is some data to write...
     this._cycle();
   }
 };
@@ -531,18 +545,28 @@ SecurePair.prototype._maybeInitFinished = function() {
 
 
 SecurePair.prototype._destroy = function() {
+  var self = this;
+
   if (!this._done) {
     this._done = true;
     this._ssl.error = null;
     this._ssl.close();
     this._ssl = null;
 
-    this.encrypted.emit('end');
-    this.encrypted.emit('close');
+    self.encrypted.writable = self.encrypted.readable = false;
+    self.cleartext.writable = self.cleartext.readable = false;
 
-    this.cleartext.emit('end');
-    this.cleartext.emit('close');
+    process.nextTick(function() {
+      self.encrypted.emit('end');
+      if (self.encrypted.onend) self.encrypted.onend();
+      self.encrypted.emit('close');
+
+      self.cleartext.emit('end');
+      if (self.cleartext.onend) self.cleartext.onend();
+      self.cleartext.emit('close');
+    });
   }
+
   this._cycle();
 };
 
