@@ -321,8 +321,8 @@ void ChildProcess::close_stdio_handles(ChildProcess *child) {
   // take some time and would deadlock if done in the main thread.
   for (int i = 0; i < 3; i++) {
     if (!child->got_custom_fds_[i]) {
-      wsa_disconnect_ex((SOCKET)child->stdio_handles_[i], NULL, 0, 0);
-      closesocket((SOCKET)child->stdio_handles_[i]);
+      shutdown(reinterpret_cast<SOCKET>(child->stdio_handles_[i]), SD_BOTH);
+      closesocket(reinterpret_cast<SOCKET>(child->stdio_handles_[i]));
     }
   }
 }
@@ -742,6 +742,10 @@ Handle<Value> ChildProcess::Spawn(const Arguments& args) {
           SOCKET_ERROR)
         wsa_perror("ioctlsocket");
 
+      // Make parent handle non-inheritable
+      if (!SetHandleInformation(parent_handle, HANDLE_FLAG_INHERIT, 0))
+        winapi_perror("SetHandleInformation");
+
       // Make child handle inheritable
       if (!SetHandleInformation(child_handle, HANDLE_FLAG_INHERIT,
           HANDLE_FLAG_INHERIT))
@@ -873,4 +877,4 @@ void ChildProcess::Initialize(Handle<Object> target) {
 
 }  // namespace node
 
-NODE_MODULE(node_child_process, node::ChildProcess::Initialize);
\ No newline at end of file
+NODE_MODULE(node_child_process, node::ChildProcess::Initialize);
